@@ -55,6 +55,7 @@ static void     gail_focus_tracker (AtkObject *object);
 
 static GtkWidget* focus_widget = NULL;
 static GtkWidget* next_focus_widget = NULL;
+static gboolean was_deselect = FALSE;
 static GtkWidget* subsequent_focus_widget = NULL;
 static GtkWidget* focus_before_menu = NULL;
 static guint focus_notify_handler = 0;    
@@ -331,21 +332,23 @@ gail_finish_select (GtkWidget *widget)
             g_signal_connect (menu_item->submenu, "map",
                               G_CALLBACK (gail_map_submenu_cb),
                               NULL);
+            return;
 
-          /*
-           * If we are waiting to report focus on a menubar or a menu item
-           * because of a previous deselect, cancel it.
-           */
-          if (focus_notify_handler &&
-              next_focus_widget &&
-              (GTK_IS_MENU_BAR (next_focus_widget) ||
-               GTK_IS_MENU_ITEM (next_focus_widget)))
-            {
-              g_source_remove (focus_notify_handler);
-              g_object_remove_weak_pointer (G_OBJECT (next_focus_widget), (gpointer *)&next_focus_widget);
-              focus_notify_handler = 0;
-            }
-          return;
+        }
+      /*
+       * If we are waiting to report focus on a menubar or a menu item
+       * because of a previous deselect, cancel it.
+       */
+      if (was_deselect &&
+          focus_notify_handler &&
+          next_focus_widget &&
+          (GTK_IS_MENU_BAR (next_focus_widget) ||
+           GTK_IS_MENU_ITEM (next_focus_widget)))
+        {
+          g_source_remove (focus_notify_handler);
+          g_object_remove_weak_pointer (G_OBJECT (next_focus_widget), (gpointer *)&next_focus_widget);
+          focus_notify_handler = 0;
+          was_deselect = FALSE;
         }
     } 
   /*
@@ -422,9 +425,12 @@ gail_deselect_watcher (GSignalInvocationHint *ihint,
       else
         {
           if (!GTK_IS_MENU_BAR (menu_shell))
-            gail_focus_notify_when_idle (menu_shell);
+            {
+              gail_focus_notify_when_idle (menu_shell);
+            }
         }
     }
+  was_deselect = TRUE;
   return TRUE; 
 }
 
@@ -533,7 +539,7 @@ gail_focus_notify_when_idle (GtkWidget *widget)
         {
           /*
            * Ignore focus request when menu item is going to be focused.
-           * See bug #123432.
+           * See bug #124232.
            */
           if (GTK_IS_MENU_ITEM (next_focus_widget) && !GTK_IS_MENU_ITEM (widget))
             return;
@@ -590,6 +596,21 @@ gail_deactivate_watcher (GSignalInvocationHint *ihint,
   if (!shell->parent_menu_shell)
     focus = focus_before_menu;
       
+  /*
+   * If we are waiting to report focus on a menubar or a menu item
+   * because of a previous deselect, cancel it.
+   */
+  if (was_deselect &&
+      focus_notify_handler &&
+      next_focus_widget &&
+      (GTK_IS_MENU_BAR (next_focus_widget) ||
+       GTK_IS_MENU_ITEM (next_focus_widget)))
+    {
+      g_source_remove (focus_notify_handler);
+      g_object_remove_weak_pointer (G_OBJECT (next_focus_widget), (gpointer *)&next_focus_widget);
+      focus_notify_handler = 0;
+      was_deselect = FALSE;
+    }
   gail_focus_notify_when_idle (focus);
 
   return TRUE; 
