@@ -583,7 +583,8 @@ gail_button_get_keybinding (AtkAction *action,
          * We look for a mnemonic on the label
          */
         GtkWidget *widget;
-        GtkWidget *child;
+        GtkWidget *label;
+        guint key_val; 
 
         button = GAIL_BUTTON (action);
         widget = GTK_ACCESSIBLE (button)->widget;
@@ -595,17 +596,47 @@ gail_button_get_keybinding (AtkAction *action,
 
         g_return_val_if_fail (GTK_IS_BUTTON (widget), NULL);
 
-        child = get_label_from_button (widget, 0, FALSE);
-        if (GTK_IS_LABEL (child))
+        label = get_label_from_button (widget, 0, FALSE);
+        if (GTK_IS_LABEL (label))
           {
-            guint key_val; 
-
-            key_val = gtk_label_get_mnemonic_keyval (GTK_LABEL (child)); 
+            key_val = gtk_label_get_mnemonic_keyval (GTK_LABEL (label)); 
             if (key_val != GDK_VoidSymbol)
               return_value = gtk_accelerator_name (key_val, GDK_MOD1_MASK);
-            g_free (button->click_keybinding);
-            button->click_keybinding = return_value;
           }
+        if (return_value == NULL)
+          {
+            /* Find labelled-by relation */
+            AtkRelationSet *set;
+            AtkRelation *relation;
+            GPtrArray *target;
+            gpointer target_object;
+
+            set = atk_object_ref_relation_set (ATK_OBJECT (action));
+            if (set)
+              {
+                relation = atk_relation_set_get_relation_by_type (set, ATK_RELATION_LABELLED_BY);
+                if (relation)
+                  {              
+                    target = atk_relation_get_target (relation);
+            
+                    target_object = g_ptr_array_index (target, 0);
+                    if (GTK_IS_ACCESSIBLE (target_object))
+                      {
+                        label = GTK_ACCESSIBLE (target_object)->widget;
+                      } 
+                  }
+                g_object_unref (set);
+              }
+
+            if (GTK_IS_LABEL (label))
+              {
+                key_val = gtk_label_get_mnemonic_keyval (GTK_LABEL (label)); 
+                if (key_val != GDK_VoidSymbol)
+                  return_value = gtk_accelerator_name (key_val, GDK_MOD1_MASK);
+              }
+          }
+        g_free (button->click_keybinding);
+        button->click_keybinding = return_value;
         break;
       }
     default:
@@ -688,10 +719,6 @@ gail_button_set_description (AtkAction      *action,
     return FALSE;
 }
 
-/*
- * We report that this object has no children
- */
-
 static gint
 gail_button_get_n_children (AtkObject* obj)
 {
