@@ -951,6 +951,7 @@ static gboolean
 gail_clist_is_row_selected (AtkTable      *table,
                             gint          row)
 {
+  GList *elem;
   GtkWidget *widget;
   GtkCList *clist;
   GtkCListRow *clist_row;
@@ -965,7 +966,10 @@ gail_clist_is_row_selected (AtkTable      *table,
   if (row < 0 || row >= clist->rows)
     return FALSE;
 
-  clist_row = ROW_ELEMENT (clist, row)->data;
+  elem = ROW_ELEMENT (clist, row);
+  if (!elem)
+    return FALSE;
+  clist_row = elem->data;
 
   return (clist_row->state == GTK_STATE_SELECTED);
 }
@@ -996,9 +1000,9 @@ gail_clist_set_caption (AtkTable      *table,
   g_value_init (&values.new_value, G_TYPE_POINTER);
   g_value_set_pointer (&values.new_value, obj->caption);
 
-  values.property_name = "accessible_table_caption";
+  values.property_name = "accessible-table-caption";
   g_signal_emit_by_name (table, 
-                         "property_change::accessible_table_caption", 
+                         "property_change::accessible-table-caption", 
                          &values, NULL);
   if (old_caption)
     g_object_unref (old_caption);
@@ -1026,9 +1030,9 @@ gail_clist_set_column_description (AtkTable      *table,
   g_value_init (&values.new_value, G_TYPE_INT);
   g_value_set_int (&values.new_value, column);
 
-  values.property_name = "accessible_table_column_description";
+  values.property_name = "accessible-table-column-description";
   g_signal_emit_by_name (table, 
-                         "property_change::accessible_table_column_description",
+                         "property_change::accessible-table-column-description",
                           &values, NULL);
 
 }
@@ -1055,9 +1059,9 @@ gail_clist_set_column_header (AtkTable      *table,
   g_value_init (&values.new_value, G_TYPE_INT);
   g_value_set_int (&values.new_value, column);
 
-  values.property_name = "accessible_table_column_header";
+  values.property_name = "accessible-table-column-header";
   g_signal_emit_by_name (table, 
-                         "property_change::accessible_table_column-header",
+                         "property_change::accessible-table-column-header",
                          &values, NULL);
 }
 
@@ -1095,9 +1099,9 @@ gail_clist_set_summary (AtkTable      *table,
   g_value_init (&values.new_value, G_TYPE_POINTER);
   g_value_set_pointer (&values.new_value, obj->summary);
 
-  values.property_name = "accessible_table_summary";
+  values.property_name = "accessible-table-summary";
   g_signal_emit_by_name (table, 
-                         "property_change::accessible_table_summary", 
+                         "property_change::accessible-table-summary", 
                          &values, NULL);
   if (old_summary)
     g_object_unref (old_summary);
@@ -1344,9 +1348,14 @@ gail_clist_set_row_data (AtkTable      *table,
     } 
   if (!found)
     {
+      GList *elem;
+
+      elem = ROW_ELEMENT (gtk_clist, row);
+      g_return_if_fail (elem != NULL);
+
       row_data = g_new (GailCListRow, 1);
       row_data->row_number = row;
-      row_data->row_data = ROW_ELEMENT (gtk_clist, row)->data;
+      row_data->row_data = elem->data;
       if (is_header)
         {
           row_data->header = header;
@@ -1367,13 +1376,13 @@ gail_clist_set_row_data (AtkTable      *table,
 
   if (is_header)
     {
-      values.property_name = "accessible_table_row_header";
-      signal_name = "property_change::accessible_table_row_header";
+      values.property_name = "accessible-table-row-header";
+      signal_name = "property_change::accessible-table-row-header";
     }
   else
     {
-      values.property_name = "accessible_table_row_description";
-      signal_name = "property_change::accessible_table_row_description";
+      values.property_name = "accessible-table-row-description";
+      signal_name = "property_change::accessible-table-row-description";
     }
   g_signal_emit_by_name (table, 
                          signal_name,
@@ -1451,12 +1460,15 @@ gail_clist_cell_data_new (GailCList     *clist,
                           gint          column,
                           gint          row)
 {
+  GList *elem;
   GailCListCellData *cell_data;
   GtkCList *gtk_clist;
   GtkCListRow *clist_row;
 
   gtk_clist = GTK_CLIST (GTK_ACCESSIBLE (clist)->widget);
-  clist_row = (GtkCListRow *) g_list_nth (gtk_clist->row_list, row)->data;
+  elem = g_list_nth (gtk_clist->row_list, row);
+  g_return_if_fail (elem != NULL);
+  clist_row = (GtkCListRow *) elem->data;
   cell_data = g_new (GailCListCellData, 1);
   cell_data->gail_cell = cell;
   cell_data->gtk_cell = &(clist_row->cell[column]);
