@@ -273,6 +273,9 @@ notify_name_change (AtkObject *atk_obj)
   label = GTK_LABEL (widget);
   gail_label = GAIL_LABEL (atk_obj);
 
+  if (gail_label->textutil == NULL)
+    return;
+
   /* Create a delete text and an insert text signal */
  
   g_signal_emit_by_name (gail_obj, "text_changed::delete", 0, 
@@ -464,6 +467,44 @@ gail_label_ref_relation_set (AtkObject *obj)
           AtkObject *accessible_array[1];
           AtkRelation* relation;
 
+          /*
+           * Handle the case where a GnomeIconEntry is specified as the 
+           * mnemonic widget. use the button which is a grandchild of the
+           * GnomeIconEntry as the mnemonic widget. See bug #133967.
+           */
+          if (!GTK_WIDGET_CAN_FOCUS (mnemonic_widget))
+            {
+              if (GTK_IS_BOX (mnemonic_widget))
+                {
+                  GList *list;
+
+                  list = gtk_container_get_children (GTK_CONTAINER (mnemonic_widget));
+                  if (g_list_length (list) == 1)
+                    {
+                      if (GTK_IS_ALIGNMENT (list->data))
+                        {
+                          GtkWidget *temp_widget;
+
+                          temp_widget = GTK_BIN (list->data)->child;
+                          if (GTK_IS_BUTTON (temp_widget))
+                            mnemonic_widget = temp_widget;
+                        }
+                      else if (GTK_IS_HBOX (list->data))
+                        {
+                          GtkWidget *temp_widget;
+
+                          temp_widget = GTK_WIDGET (list->data);
+                          g_list_free (list);
+                          list = gtk_container_get_children (GTK_CONTAINER (temp_widget));
+                          if (GTK_IS_COMBO (list->data))
+                            {
+                              mnemonic_widget = GTK_COMBO (list->data);
+                            }
+                        }
+                    }
+                  g_list_free (list);
+                }
+            }
           accessible_array[0] = gtk_widget_get_accessible (mnemonic_widget);
           relation = atk_relation_new (accessible_array, 1,
                                        ATK_RELATION_LABEL_FOR);
