@@ -354,8 +354,6 @@ is_attached_menu_window (GtkWidget *widget)
           GTK_IS_OPTION_MENU (attach) ||
           GTK_IS_BUTTON (attach))
         ret = TRUE;
-      else if (attach)
-        g_warning ("Unexpected attach type %s\n", G_OBJECT_TYPE_NAME (attach));
     }
   return ret;
 }
